/*!
 * \file defs.h
 *
 * \brief Miscellaneous definitions.
 *
 * This file sets default values for any constants that should be in include
 * files but aren't, or that have wacky values.
 *
 * Blitzed Services copyright (c) 2000-2002 Blitzed Services team
 *     E-mail: services@lists.blitzed.org
 * Based on ircservices-4.4.8:
 * copyright (c) 1996-1999 Andrew Church.
 *     E-mail: <achurch@dragonfire.net>
 * copyright (c) 1999-2000 Andrew Kempe.
 *     E-mail: <theshadow@shadowfire.org>
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 */

/*************************************************************************/

#ifndef NAME_MAX
# define NAME_MAX 255
#endif

#ifndef BUFSIZ
# define BUFSIZ 256
#else
# if BUFSIZ < 256
#  define BUFSIZ 256
# endif
#endif

/* Length of an array: */
#define lenof(a)	(sizeof(a) / sizeof(*(a)))

/* Telling compilers about printf()-like functions: */
#ifdef __GNUC__
# define FORMAT(type,fmt,start) __attribute__((format(type,fmt,start)))
#else
# define FORMAT(type,fmt,start)
#endif

/* Stuff to shut up warnings about rcsid being unused. */
#define USE_VAR(var)	static char sizeof##var = sizeof(sizeof##var) + sizeof(var)
#define RCSID(x)	static char rcsid[] = x; USE_VAR(rcsid);

/* stick the file name and line number into the various MySQL query functions */
#define smysql_bulk_query(m, f, r, q, args...)  real_smysql_bulk_query(__FILE__, __LINE__, (m), (f), (r), (q) , ##args)
#define mysql_simple_query(m, e, q, args...)    real_mysql_simple_query(__FILE__, __LINE__, (m), (e), (q) , ##args)
#define smysql_query(m, q, args...)             real_smysql_query(__FILE__, __LINE__, (m), (q) , ##args)
#define read_lock_table(m, t)               real_read_lock_table(__FILE__, __LINE__, (m), (t))
#define write_lock_table(m, t)              real_write_lock_table(__FILE__, __LINE__, (m), (t))
#define smysql_cleanup(m)                   real_smysql_cleanup(__FILE__, __LINE__, (m))

/*************************************************************************/
